#include "StdAfx.h"
#include "GameTalkAPIGlobal.h"
#include "MessengerActor.h"
#include "ProfileImageActor.h"
#include "ProfileImageSender.h"
#include "UtilFunction.h"
#include "ProfileImageActor.h"
#include "_t_UTF8.h"
#include "ConvertUTF.h"
#include "GBStream.h"
#include "NpProtocolCodeDefine.h"

#ifdef __USE_PROFILE_IMG_BY_P2P__

#define SEND_FRAME_SIZE         4096

CProfileImageSender::CProfileImageSender(CProfileImageActor *pActor) :
m_pProfileImageActor(pActor), 
m_pSession(NULL),
m_hFile(NULL),
m_dwWritten(0),
m_dwFileSize(0),
m_bStartSend(false),
m_eSizeType(profileOriginalSize)
{
	m_strFileName  = GT_GLOBAL->GetEnvironment().strDefaultProfileImageFile;
	
	if(m_strFileName.empty())
		GT_ASSERT(false);
}


CProfileImageSender::~CProfileImageSender(void)
{
	if(m_hFile)
		CloseHandle(m_hFile);
	if(m_pSession)
		m_pSession->Delete();
}

void CProfileImageSender::SetStreamSession(INpStreamSession * pSession, INpPacket * pPacket)
{
	m_pSession = pSession;
	pSession->SetEvents(this);
	NpStreamSession_OnRecvPacket(pSession, pPacket);
}


void CProfileImageSender::_Delete()
{
	if(m_pSession)
		m_pSession->CloseSession();

	m_pProfileImageActor->DeleteSender(this);

	// WARNING!!
	// Never coding Below this line!
}


bool CProfileImageSender::_SendCRC()
{
	DWORD dwCRC = 0;

	// REFACTORING..  ADDED FOLLOWING CODES..
	// KEH_MERGE_5856_8500
	if(!m_pProfileImageActor->GetMessengerActor())
		return false;

	WSTRING strUserID = m_pProfileImageActor->GetMessengerActor()->GetUserID();

	m_strFileName = CProfileImageActor::GetProfileImageFilePathBySizeType(strUserID.c_str(), m_eSizeType);
	////////////////////////////////////////////

	m_dwFileSize = CProfileImageActor::GetFileCRC(m_strFileName.c_str(), &dwCRC);

	if(m_dwFileSize == 0)
		return false;

	USES_UTF8_CONVERSION
	WSTRING szShortFileName = FileUtil::ExtractFileName(m_strFileName.c_str());
	nfc::CGBStream stream;
	stream.CreateStream();
	stream.Write_U4(dwCRC);
	stream.Write_U8SS((const nfc::UTF8 *)W2U8(szShortFileName.c_str()));
	int nTemp = stream.GetDataLen();

	m_pSession->SendNpPacket_M6V2(RESPONSE(REQ_PFIMG_CKSUM), (USHORT)stream.GetDataLen(), (LPCSTR)stream.GetData());
	return true;
}


bool CProfileImageSender::_StartSend()
{
	//CGTLifeLog log(__FUNCTION__);

	if(m_hFile)
		return false;

	m_hFile = CreateFile(m_strFileName.c_str(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(m_hFile == INVALID_HANDLE_VALUE)
		return false;

	m_bStartSend = true;
	NpStreamSession_OnSendReady(m_pSession);
	return true;
}




//------------------------------------------------------------------------------------------------
//
// _INpStreamSessionEvents 
//
//------------------------------------------------------------------------------------------------

void CALLBACK CProfileImageSender::NpStreamSession_OnConnect(INpStreamSession * /*pSession*/, int /*nResultCode*/, int /*nErrorCode*/)
{
}

void CALLBACK CProfileImageSender::NpStreamSession_OnClose(INpStreamSession * /*pSession*/, int /*nSocketError*/, bool /*bRecvInvalidPacket*/)
{
	_Delete();
}

void CALLBACK CProfileImageSender::NpStreamSession_OnRecvPacket(INpStreamSession * /*pSession*/, INpPacket * pPacket)
{
	switch(pPacket->M6V2()->usCMMD)
	{
	case REQ_PFIMG_CKSUM:
		{
			// REFACTORING..  ADDED FOLLOWING CODES..
			// KEH_MERGE_5856_8500
			nfc::CGBStream stream;
			WORD nSizeType = 0;
			try
			{
				stream.UseExternalBuffer(pPacket->BodyData(), pPacket->BodyLength());
				nSizeType = stream.Read_U2();

				m_eSizeType = (GTProfileImageSizeType)nSizeType;
			}
			catch(nfc::CGBStreamException e)
			{
			}
			////////////////////////////////////////

			if(_SendCRC() == false)
				_Delete();
			break;
		}

	case REQ_PFIMG_REQDN:
		if(_StartSend() == false)
			_Delete();
		break;
	}
}

void CALLBACK CProfileImageSender::NpStreamSession_OnSendReady(INpStreamSession *pSession)
{
	char buf[SEND_FRAME_SIZE + 1];
	if(m_bStartSend)
	{
		DWORD dwRead = 0;

		if(ReadFile(m_hFile, buf + 1, SEND_FRAME_SIZE, &dwRead, NULL))
		{
			if(m_dwWritten + dwRead >= m_dwFileSize)
			{
				buf[0] = 1; // Ϸ...
				m_bStartSend = false;
			} else
				buf[0] = 0; // ...
			pSession->SendNpPacket_M6V2(RESPONSE(REQ_PFIMG_REQDN), (USHORT)dwRead + 1, buf);

			m_dwWritten += dwRead;
		} else
			_Delete();
	}
}

#endif
